/***
 * Juliac
 * Copyright (C) 2007-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Romain Rouvoy
 */

package org.objectweb.fractal.juliac.opt.mergeall;

import java.lang.reflect.Modifier;

import org.objectweb.fractal.juliac.api.ClassGeneratorItf;
import org.objectweb.fractal.juliac.visit.BlockSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.FileSourceCodeVisitor;

import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ClassGenerator;

/**
 * A {@link ClassGenerator} for generating {@link RuntimeException} classes.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Romain Rouvoy <Romain.Rouvoy@univ-lille1.fr>
 */
public class RuntimeExceptionClassGenerator implements ClassGeneratorItf {
    
    private String targetClassName;
    
    /**
     * @param targetClassName  the name of the exception class
     */
    public RuntimeExceptionClassGenerator( String targetClassName ) {
        this.targetClassName = targetClassName;
    }
    
    public String getTargetClassName() { return targetClassName;}

    public void generate( FileSourceCodeVisitor fv ) {
        
        int lastdot = targetClassName.lastIndexOf('.');
        String packagename =
        	(lastdot==-1) ? null : targetClassName.substring(0,lastdot);
        
        if( packagename != null ) {
            fv.visitPackageName(packagename);
        }
        
        ClassSourceCodeVisitor cv = fv.visitPublicClass();
        generate(cv);
    }
    
    public void generate( ClassSourceCodeVisitor cv ) {

        int lastdot = targetClassName.lastIndexOf('.');
        String classname = targetClassName.substring(lastdot+1);
        
        cv.visit(Modifier.PUBLIC, classname, null, "RuntimeException", null);
        
        BlockSourceCodeVisitor mv =
            cv.visitConstructor(
                Modifier.PUBLIC, null, new String[]{"String msg"}, null );
        mv.visitln("    super(msg);");
        mv.visitEnd();
        
        cv.visitEnd();
    }
}
